//===========================================================================
// @(#) $Name:  $
// @(#) $Id: DwmWWWGalleryUploadResource.cc,v 1.5 2015/05/02 08:35:41 dwm Exp $
//===========================================================================
//  Copyright (c) Daniel W. McRobb 2015
//  All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions
//  are met:
//
//  1. Redistributions of source code must retain the above copyright
//     notice, this list of conditions and the following disclaimer.
//  2. Redistributions in binary form must reproduce the above copyright
//     notice, this list of conditions and the following disclaimer in the
//     documentation and/or other materials provided with the distribution.
//  3. The names of the authors and copyright holders may not be used to
//     endorse or promote products derived from this software without
//     specific prior written permission.
//
//  IN NO EVENT SHALL DANIEL W. MCROBB BE LIABLE TO ANY PARTY FOR
//  DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
//  INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE,
//  EVEN IF DANIEL W. MCROBB HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
//  DAMAGE.
//
//  THE SOFTWARE PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND
//  DANIEL W. MCROBB HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
//  UPDATES, ENHANCEMENTS, OR MODIFICATIONS. DANIEL W. MCROBB MAKES NO
//  REPRESENTATIONS AND EXTENDS NO WARRANTIES OF ANY KIND, EITHER
//  IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
//  WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE,
//  OR THAT THE USE OF THIS SOFTWARE WILL NOT INFRINGE ANY PATENT,
//  TRADEMARK OR OTHER RIGHTS.
//===========================================================================

//---------------------------------------------------------------------------
//!  \file DwmWWWGalleryUploadResource.cc
//!  \brief NOT YET DOCUMENTED
//---------------------------------------------------------------------------

extern "C" {
  #include <sys/types.h>
  #include <sys/stat.h>
  #include <unistd.h>
}

#include <iostream>
#include <sstream>

#include "DwmSysLogger.hh"
#include "DwmWWWGalleryDropUpload.hh"
#include "DwmWWWGalleryUploadResource.hh"

using namespace std;

namespace Dwm {

  namespace WWW {

    namespace Gallery {

      //----------------------------------------------------------------------
      //!  
      //----------------------------------------------------------------------
      UploadResource::UploadResource(DropUpload *dropUpload)
          : Wt::WResource(dropUpload), _dropUpload(dropUpload),
            _uploadedFiles(), _droppedURIs()
      {
      }

      //----------------------------------------------------------------------
      //!  
      //----------------------------------------------------------------------
      UploadResource::~UploadResource()
      {
        if (! _uploadedFiles.empty()) {
          //  remove spool files.
          auto  it = _uploadedFiles.begin();
          for ( ; it != _uploadedFiles.end(); ++it) {
            unlink(it->spoolFileName().c_str());
          }
        }
      }
      
      //----------------------------------------------------------------------
      //!  
      //----------------------------------------------------------------------
      void
      UploadResource::handleRequest(const Wt::Http::Request & request,
                                    Wt::Http::Response & response)
      {
        Syslog(LOG_INFO, "%u files uploaded", request.uploadedFiles().size());
        //  Handle file uploads.
        if (! request.uploadedFiles().empty()) {
          auto  it = request.uploadedFiles().begin();
          for ( ; it != request.uploadedFiles().end(); ++it) {
            _uploadedFiles.push_back(it->second);
            it->second.stealSpoolFile();
          }
          _dropUpload->filesUploaded();
        }

        //  Handle URIs.  This is just a string, it's up to the server to
        //  decide what to do with it.  In my use cases, the server side
        //  will fetch the data at the URIs.
        auto uri = request.getParameterMap().find("uri");
        if (uri != request.getParameterMap().end()) {
          auto  urit = uri->second.begin();
          for ( ; urit != uri->second.end(); ++urit) {
            Syslog(LOG_INFO, "uri: %s", urit->c_str());
            _droppedURIs.push_back(*urit);
          }
          _dropUpload->signalURIDropped();
        }

        return;
      }
      
      //----------------------------------------------------------------------
      //!  
      //----------------------------------------------------------------------
      bool UploadResource::getNextFile(Wt::Http::UploadedFile & uploadedFile)
      {
        bool  rc = false;
        if (! _uploadedFiles.empty()) {
          uploadedFile = _uploadedFiles.front();
          _uploadedFiles.pop_front();
          rc = true;
        }
        return rc;
      }

      //----------------------------------------------------------------------
      //!  
      //----------------------------------------------------------------------
      bool UploadResource::getNextURI(string & uri)
      {
        bool  rc = false;
        if (! _droppedURIs.empty()) {
          uri = _droppedURIs.front();
          _droppedURIs.pop_front();
          rc = true;
        }
        return rc;
      }
      
    }  // namespace Gallery

  }  // namespace WWW

}  // namespace Dwm
